/*
 * Copyright (c) 2007 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.domain;

import java.io.IOException;

/**
 * @author klaus
 * 
 */
public interface IIndividuum extends IProblemState, Comparable<IIndividuum>
{
	/**
	 * Calculates the fitness of the individuum
	 */
	float calculateFitness();

	/**
	 * @return the fitness of this individuum as a positive float value
	 */
	float getFitness();

	/**
	 * @return the age of this individuum
	 */
	public int getAge();

	/**
	 * Increments the age of this individuum by one (generation)
	 */
	public void incrementAge();

	/**
	 * @return the chromosom information that represents the individuum
	 */
	public float[] getChromosom();

	/**
	 * @param criteria a float value that is used by the selection strategy
	 */
	public void setSelectionCriteria(float criteria);

	/**
	 * @return a float value that is used by the selection strategy
	 */
	public float getSelectionCriteria();

	/**
	 * Mutates the ith gene by replacing it with a random allowed gene
	 * @param i the index of the gene to mutate
	 */
	void mutate(int i);

	/**
	 * @param other the individuum to compare against
	 * @return the relative number of genes that differ for this and the passed
	 *         individuum
	 */
	float getRelativeDiversity(IIndividuum other);

	/**
	 * Checks the consistency of this state to be defined by each problem state
	 * separately
	 */
	public void checkConsistency();

	/**
	 * Writes this individuum to the passed file
	 * @param filename path and name of the file to write
	 * @throws IOException if writing is not possible
	 */
	public void writeToFile(String filename) throws IOException;
}
