/*
 * Copyright (c) 2008 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.domain.model;

import java.util.Comparator;
import java.util.List;

import kdo.domain.IGeneticProblem;
import kdo.domain.IIndividuum;
import kdo.domain.IOperator;
import kdo.domain.IProblemState;
import kdo.util.IRandomSource;

/**
 * Base class for Genetic Problems. Since it does not add any implementation
 * needs to its subclasses this can also be used for normal problems.
 * @author dorer
 */
public abstract class GeneticProblem extends Problem implements IGeneticProblem
{
	/** source for pseudo random numbers */
	protected IRandomSource rand;

	/**
	 * Default constructor
	 * @param initialState the start state of the problem
	 * @param operators the operators that are available in all states
	 * @param rand the source for random numbers to use
	 */
	public GeneticProblem(IProblemState initialState, List<IOperator> operators,
		IRandomSource rand)
	{
		super(initialState, operators);
		this.rand = rand;
	}

	/*
	 * (non-Javadoc)
	 * @see domain.IGeneticProblem#getRandomIndividuum()
	 */
	public IIndividuum getRandomIndividuum()
	{
		return (IIndividuum) getRandomState();
	}

	/*
	 * (non-Javadoc)
	 * @see domain.IGeneticProblem#getIndividuumComparator()
	 */
	public Comparator<IIndividuum> getIndividuumComparator()
	{
		return new Individuum.IndividuumFitnessComparator();
	}
}
