/*
 * Lecture symbolic AI, FH-Furtwangen
 * Original Author: Klaus Dorer
 */

package kdo.domain.model;

import java.util.List;

import kdo.domain.IOperator;
import kdo.domain.IProblem;
import kdo.domain.IProblemState;

/**
 * Represents a domain problem containing the initial state, operator
 * descriptions a goal test and cost functions
 */
public abstract class Problem implements IProblem
{
	/** the initial state from which to start the search */
	private IProblemState initialState;

	/** the current state of the problem */
	protected IProblemState currentState;

	/** the list of operators allowed for an agent */
	protected List<IOperator> operators;

	/**
	 * Constructor creating a new domain problem instance.
	 * @param initialState the starting state from which to start the search
	 * @param operators a list of operators allowed for an agent
	 */
	public Problem(IProblemState initialState, List<IOperator> operators)
	{
		ProblemState.setProblem(this);
		this.initialState = initialState;
		this.operators = operators;
		this.currentState = initialState;
	}

	/**
	 * @see kdo.domain.IProblem#getInitialState()
	 */
	public IProblemState getInitialState()
	{
		return initialState;
	}

	/**
	 * @see kdo.domain.IProblem#setInitialState(kdo.domain.IProblemState)
	 */
	public void setInitialState(IProblemState state)
	{
		initialState = state;
	}

	/**
	 * @see kdo.domain.IProblem#getCurrentState()
	 */
	public IProblemState getCurrentState()
	{
		return currentState;
	}

	/**
	 * @see kdo.domain.IProblem#setCurrentState(kdo.domain.IProblemState)
	 */
	public void setCurrentState(IProblemState state)
	{
		currentState = state;
	}

	/*
	 * The default implementation does not return a random state, but the initial
	 * state
	 * @see domain.IProblem#getRandomState()
	 */
	public IProblemState getRandomState()
	{
		return initialState;
	}

	/*
	 * (non-Javadoc)
	 * @see domain.IProblemState#getRandomOperator(int)
	 */
	public IOperator getRandomOperator(int neighborhoodRadius)
	{
		// just implemented until all domains support this
		return null;
	}

	/**
	 * @return the operators of this domain (for domains that have all operators
	 *         always available)
	 */
	public List<IOperator> getOperators()
	{
		return operators;
	}
}
