/*
 * Lecture symbolic AI, FH-Furtwangen
 * Original Author: Klaus Dorer
 */
package kdo.search;

import kdo.domain.IProblem;
import kdo.domain.vierGewinnt.ComputerPlayer;
import kdo.domain.vierGewinnt.Player;
import kdo.domain.vierGewinnt.VierGewinnt;
import kdo.domain.vierGewinnt.VierGewinntStrategie;
import kdo.search.impl.GeneralTwoPlayerSearch;
import kdo.search.representation.Node;
import kdo.search.strategy.ITwoPlayerSearchStrategy;
import kdo.search.strategy.twoPlayer.AlphaBetaSearch;

/**
 * This is the main class that allows to select the domain which should be used
 * @author Klaus Dorer
 */
public class VierGewinntLearn
{
	/**
	 * The main method to start the search or two player game selected
	 * @param args the list of command line options
	 */
	public static void main(String[] args)
	{
		// get the domain and strategy
		ITwoPlayerSearchStrategy twoPlayerStrategy1 = new AlphaBetaSearch();
		ITwoPlayerSearchStrategy twoPlayerStrategy2 = new AlphaBetaSearch();
		Player player1 = new ComputerPlayer("player 1", (byte) 1,
			twoPlayerStrategy1);
		Player player2 = new ComputerPlayer("player 2", (byte) 2,
			twoPlayerStrategy1);
		IProblem domain = VierGewinnt.getInstance(1, player1, player2);

		long startTime = System.currentTimeMillis();

		// now the call to start the game
		int[][] stats = new int[8][3];
		int result;
		for (int depth = 1; depth < 7; depth++) {
			VierGewinntStrategie.depthMediumMax = depth;
			for (int i = 1; i < 7; i++) {
				VierGewinntStrategie.depthMediumMin = i;
				result = GeneralTwoPlayerSearch.playGame(domain,
					twoPlayerStrategy1, twoPlayerStrategy2);
				if (result < 3) {
					stats[depth][result]++;
				}
			}
		}

		long stopTime = System.currentTimeMillis();

		for (int depth = 1; depth < 8; depth++) {
			System.out.println("draws-wins-losses: " + stats[depth][0] + "-"
				+ stats[depth][1] + "-" + stats[depth][2]);
		}
		System.out.println("Time for search: " + (stopTime - startTime));
		System.out.println("Nodes created: " + Node.getNumberOfNodes());
	}
}
