/*
 * Copyright (c) 2007 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.search.strategy.local.genetic;

import java.util.Comparator;

import kdo.domain.IIndividuum;

/**
 * @author klaus
 * 
 */
public interface IPopulation
{
	/**
	 * Runs the population through the passed number of generations
	 * @param generations the number of generations to run through
	 * @param maxRuntime the maximal time to breed in ms
	 * @return the best individuum of all breeded individuums in all generations
	 */
	IIndividuum breed(int generations, long maxRuntime);

	/**
	 * @return the best individuum of the population according to the fitness
	 *         function
	 */
	IIndividuum getBestIndividuum();

	/**
	 * @param the comparator to use to compare individuums
	 * @return the best individuum of the population according to the passed
	 *         comparator
	 */
	IIndividuum getBestIndividuum(Comparator<IIndividuum> comparator);

	/**
	 * Creates a randomly created individuum from the application domain this
	 * population is based on. The individuum is not added to the population.
	 * @return a randomly created individuum from the application domain this
	 *         population is based on
	 */
	public IIndividuum createRandomIndividuum();

	/**
	 * @return the number of individuums in the population
	 */
	int getSize();

	/**
	 * @return the number of generations this population has been breeded
	 */
	int getGenerations();

	/**
	 * @return the average diversity of all genes in the population
	 */
	public float getAverageDiversity();

	/**
	 * @return the time that breeding has taken until now
	 */
	public long getRuntime();

	/**
	 * Interrupts breeding once the next generation is to be created
	 */
	public void interrrupt();
}
