/*
 * Copyright (c) 2007 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.search.strategy.local.genetic.impl;

import kdo.domain.IGeneticProblem;
import kdo.domain.IIndividuum;
import kdo.domain.IProblemState;
import kdo.search.impl.StrategyBase;
import kdo.search.strategy.ILocalSearchStrategy;
import kdo.search.strategy.local.genetic.IPopulation;
import kdo.search.strategy.local.genetic.representation.Population;
import kdo.util.observer.IObserver;
import kdo.util.observer.IPublishSubscribe;

/**
 * @author klaus
 * 
 */
public class GeneticSearch extends StrategyBase implements
	ILocalSearchStrategy, IObserver<IPopulation>
{
	private final GeneticSearchParameter parameters;

	private IPopulation population;

	/** the observer manager to be called on model changes */
	private IPublishSubscribe<IPopulation> observerManager;

	public GeneticSearch(GeneticSearchParameter parameterObject)
	{
		super("GeneticSearch", parameterObject.getRandomSource());
		parameters = parameterObject;
	}

	/*
	 * (non-Javadoc)
	 * @see search.strategy.ILocalSearchStrategy#search(domain.IProblemState)
	 */
	public IProblemState search(IProblemState initialState)
	{
		// We do not make use of the provided initial State
		// TODO we could use the initial state to create random variations of it
		// for the initial population
		population = new Population((IGeneticProblem) initialState.getProblem(),
			parameters.getGenders(), parameters.getPopulationSize(), parameters
				.getSelectionStrategy(), parameters.getReproductionStrategy(),
			parameters.getMutationStrategy(), parameters.getOldToNew(), parameters
				.getParentsPerIndividuum(), this);

		IIndividuum best = population.breed(parameters.getGenerations(),
			parameters.getMaxRuntime());
		return (IProblemState) best;
	}

	/*
	 * (non-Javadoc)
	 * @see search.strategy.local.genetic.StrategyBase#getName()
	 */
	@Override
	public String getName()
	{
		StringBuffer result = new StringBuffer(500);
		result.append(super.getName());
		result.append(" generations: ").append(parameters.getGenerations());
		result.append(" population: ").append(population);
		result.append(" " + parameters.getReproductionStrategy().getName());
		result.append(" " + parameters.getSelectionStrategy().getName());
		result.append(" " + parameters.getMutationStrategy().getName());
		return result.toString();
	}

	/**
	 * Called to indicate a state/utility change
	 */
	public void update(IPopulation population)
	{
		if (getObserverManager() != null) {
			getObserverManager().onStateChange(population);
		}
	}

	/**
	 * Call to interrupt the search for better solutions
	 */
	public void interrupt()
	{
		if (population != null) {
			population.interrrupt();
		} else {
			System.out.println("Interrupted too early, try again");
		}
	}

	/*
	 * (non-Javadoc)
	 * @see search.strategy.IStrategy#setObserverManager(kdo.util.observer.IPublishSubscribe)
	 */
	public void setObserverManager(IPublishSubscribe<IPopulation> observer)
	{
		observerManager = observer;
	}

	/**
	 * @return the observerManager
	 */
	public IPublishSubscribe<IPopulation> getObserverManager()
	{
		return observerManager;
	}
}
