package kdo.search.strategy.local.genetic.impl;

import kdo.search.impl.SearchParameters;
import kdo.search.strategy.local.genetic.IMutation;
import kdo.search.strategy.local.genetic.IReproduction;
import kdo.search.strategy.local.genetic.ISelection;
import kdo.util.IRandomSource;

public class GeneticSearchParameter extends SearchParameters
{
	public static final String MIN_DIVERSITY_MONTE_CARLO_SELECTION = "Min Diversity";

	public static final String MONTE_CARLO_SELECTION = "MonteCarlo";

	public static final String RANK_SELECTION = "Rank";

	public static final String ELITE_SELECTION = "Elite";

	public static final String SINGLE_CROSSOVER_RECOMBINATION = "Single crossover";

	public static final String MULTI_CROSSOVER_RECOMBINATION = "Multi crossover";

	public static final String RANDOM_MUTATION = "Random";

	private int generations;

	private int populationSize;

	private int genders;

	private float oldToNew;

	private int parentsPerIndividuum;

	private float individuumMutationProbability;

	private float geneMutationProbability;

	private float eliteSelectionRatio;

	private ISelection selectionStrategy;

	private IMutation mutationStrategy;

	private IReproduction reproductionStrategy;

	public GeneticSearchParameter(IRandomSource random)
	{
		super(random, "40");
		generations = 300;
		populationSize = 240;
		genders = 2;
		parentsPerIndividuum = 2;
		oldToNew = 0.1f;
		individuumMutationProbability = 0.0f;// 0.0 to switch off
		geneMutationProbability = 0.05f;
		eliteSelectionRatio = 0.5f;
		selectionStrategy = null;
		reproductionStrategy = null;
		mutationStrategy = null;
	}

	public int getGenerations()
	{
		return generations;
	}

	public void setGenerations(int generations)
	{
		this.generations = generations;
	}

	public int getPopulationSize()
	{
		return populationSize;
	}

	public void setPopulationSize(int population)
	{
		this.populationSize = population;
	}

	public int getGenders()
	{
		return genders;
	}

	public void setGenders(int genders)
	{
		this.genders = genders;
	}

	public float getOldToNew()
	{
		return oldToNew;
	}

	public void setOldToNew(float oldToNew)
	{
		this.oldToNew = oldToNew;
	}

	public int getParentsPerIndividuum()
	{
		return parentsPerIndividuum;
	}

	public void setParentsPerIndividuum(int parentsPerIndividuum)
	{
		this.parentsPerIndividuum = parentsPerIndividuum;
	}

	public float getIndividuumMutationProbability()
	{
		return individuumMutationProbability;
	}

	public void setIndividuumMutationProbability(
		float individuumMutationProbability)
	{
		this.individuumMutationProbability = individuumMutationProbability;
	}

	public float getGeneMutationProbability()
	{
		return geneMutationProbability;
	}

	public void setGeneMutationProbability(float geneMutationProbability)
	{
		this.geneMutationProbability = geneMutationProbability;
	}

	/**
	 * @return the selectionStrategy
	 */
	public ISelection getSelectionStrategy()
	{
		return selectionStrategy;
	}

	/**
	 * @param selectionStrategy the selectionStrategy to set
	 */
	public void setSelectionStrategy(ISelection selectionStrategy)
	{
		this.selectionStrategy = selectionStrategy;
	}

	/**
	 * @return the mutationStrategy
	 */
	public IMutation getMutationStrategy()
	{
		return mutationStrategy;
	}

	/**
	 * @param mutationStrategy the mutationStrategy to set
	 */
	public void setMutationStrategy(IMutation mutationStrategy)
	{
		this.mutationStrategy = mutationStrategy;
	}

	/**
	 * @return the reproductionStrategy
	 */
	public IReproduction getReproductionStrategy()
	{
		return reproductionStrategy;
	}

	/**
	 * @param reproductionStrategy the reproductionStrategy to set
	 */
	public void setReproductionStrategy(IReproduction reproductionStrategy)
	{
		this.reproductionStrategy = reproductionStrategy;
	}

	public double getIndividuumsPerGender()
	{
		return (getPopulationSize() / getGenders());
	}

	/**
	 * @return the eliteSelectionRatio
	 */
	public float getEliteSelectionRatio()
	{
		return eliteSelectionRatio;
	}

	/**
	 * @param eliteSelectionRatio the eliteSelectionRatio to set
	 */
	public void setEliteSelectionRatio(float eliteSelectionRatio)
	{
		this.eliteSelectionRatio = eliteSelectionRatio;
	}
}